import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:sk_base_mobile/app_theme.dart';
import 'package:sk_base_mobile/config.dart';
import 'package:sk_base_mobile/constants/bg_color.dart';
import 'package:sk_base_mobile/screens/inventory_inout/inventory_inout_controller.dart';
import 'package:sk_base_mobile/util/util.dart';
import 'package:sk_base_mobile/widgets/fade_in_cache_image.dart';

import '../../../constants/enum.dart';

class InventoryInoutCard extends StatelessWidget {
  InventoryInoutCard({super.key, required this.index});
  final int index;
  final controller = Get.find<InventoryInoutController>();
  @override
  Widget build(BuildContext context) {
    return InkWell(
      onTap: () {
        controller
            .showInventoryInoutInfoDialog(controller.inoutList[index].id!);
      },
      child: Stack(
        children: [
          Container(
            margin: EdgeInsets.symmetric(
                vertical: ScreenAdaper.height(10),
                horizontal: ScreenAdaper.width(10)),
            padding: EdgeInsets.symmetric(
                horizontal: ScreenAdaper.width(defaultPadding),
                vertical: ScreenAdaper.height(defaultPadding)),
            decoration: BoxDecoration(
                boxShadow: [
                  BoxShadow(
                      color: AppTheme.barrierColor.withOpacity(0.2),
                      offset: Offset(0, ScreenAdaper.height(5)),
                      blurRadius: ScreenAdaper.sp(10)),
                ],
                color: AppTheme.nearlyWhite,
                borderRadius: BorderRadius.circular(ScreenAdaper.sp(30))),
            child: Row(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: [
                buildImage(),
                Expanded(child: buildContent()),
              ],
            ),
          ),
          Positioned(
            right: ScreenAdaper.width(5),
            top: ScreenAdaper.height(5),
            child: buildInOrOut(),
          )
        ],
      ),
    );
  }

  Widget buildContent() {
    return Column(
      mainAxisAlignment: MainAxisAlignment.start,
      crossAxisAlignment: CrossAxisAlignment.start,
      children: [
        Row(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: [
            Expanded(
                child: Text(
              controller.inoutList[index].product?.name ?? '-',
              overflow: TextOverflow.ellipsis,
              style: TextStyle(
                  color: Colors.black,
                  fontWeight: FontWeight.bold,
                  fontSize: ScreenAdaper.height(25)),
            )),
          ],
        ),
        Text(
          controller.inoutList[index].product?.company?.name ?? '-',
          style: TextStyle(
              color: AppTheme.grey, fontSize: ScreenAdaper.height(20)),
        ),
        SizedBox(
          height: ScreenAdaper.height(5),
        ),
        Row(
          children: [
            Expanded(
              child: Text(
                '${controller.inoutList[index].project?.name}',
                style: TextStyle(
                    color: AppTheme.grey, fontSize: ScreenAdaper.height(20)),
              ),
            ),
          ],
        ),
        SizedBox(
          height: ScreenAdaper.height(20),
        ),
        Row(
          children: [
            Expanded(
              child: Text(
                '${controller.inoutList[index].quantity} ${controller.inoutList[index].product?.unit?.label ?? '-'}',
                style: TextStyle(
                    fontSize: ScreenAdaper.height(20),
                    fontWeight: FontWeight.w600),
              ),
            ),
            Text(
              '${SkDateUtil.format(controller.inoutList[index].time!, formats: [
                    'HH',
                    ':',
                    "nn"
                  ])} ${controller.inoutList[index].agent ?? '-'}',
              style: TextStyle(
                  fontSize: ScreenAdaper.height(20),
                  fontWeight: FontWeight.w600),
            ),
          ],
        )
      ],
    );
  }

  Widget buildImage() {
    return Container(
        margin: EdgeInsets.only(right: ScreenAdaper.width(defaultPadding)),
        child: ClipRRect(
          borderRadius: BorderRadius.circular(ScreenAdaper.sp(15)),
          child: FadeInCacheImage(
              width: ScreenAdaper.height(150),
              height: ScreenAdaper.height(150),
              url: controller.inoutList[index].files.isNotEmpty
                  ? '${GloablConfig.OSS_URL}${controller.inoutList[index].files[0].path}'
                  : ''),
        ));
  }

  Widget buildInOrOut() {
    final bgColor =
        controller.inoutList[index].inOrOut == InventoryInOrOutEnum.In
            ? AppTheme.snackbarSuccessBackgroudColor
            : AppTheme.snackbarErrorBackgroudColor;
    final textInOut =
        controller.inoutList[index].inOrOut == InventoryInOrOutEnum.In
            ? '入'
            : '出';
    return Container(
      alignment: Alignment.center,
      height: ScreenAdaper.height(40),
      width: ScreenAdaper.height(40),
      decoration: BoxDecoration(
          shape: BoxShape.circle,
          gradient: LinearGradient(
              begin: Alignment.centerLeft,
              end: Alignment.centerRight,
              colors: [
                bgColor.withOpacity(0.6),
                bgColor,
              ]),
          boxShadow: [
            BoxShadow(
                color: lightOrange,
                offset: Offset(0, ScreenAdaper.height(10)),
                blurRadius: ScreenAdaper.sp(10))
          ]),
      child: Text(
        textInOut,
        style: TextStyle(
            fontSize: ScreenAdaper.height(25),
            color: AppTheme.nearlyWhite,
            fontWeight: FontWeight.w600),
      ),
    );
  }
}
