import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:sk_base_mobile/router/router.util.dart';
import 'package:sk_base_mobile/util/logger_util.dart';
import 'package:sk_base_mobile/util/screen_adaper_util.dart';

// ignore: must_be_immutable
class SkBaseDatePicker extends StatelessWidget {
  final Function(String)? onDateTimeChanged;
  late final FixedExtentScrollController yearController;
  late final FixedExtentScrollController monthController;
  late final FixedExtentScrollController dayController;
  DateTime? initialDate = DateTime.now();
  DateTime? endDate = DateTime.now();
  SkBaseDatePicker(
      {super.key, this.onDateTimeChanged, this.initialDate, this.endDate});

  @override
  Widget build(BuildContext context) {
    yearController = FixedExtentScrollController(
        initialItem: (initialDate ?? DateTime.now()).year - 2000);
    monthController = FixedExtentScrollController(
        initialItem: (initialDate ?? DateTime.now()).month - 1);
    dayController = FixedExtentScrollController(
        initialItem: (initialDate ?? DateTime.now()).day - 1);
    return Container(
      height: ScreenAdaper.height(250),
      color: Colors.white,
      child: Column(
        children: [
          Row(
            children: [
              TextButton(
                  onPressed: () {
                    RouterUtil.back();
                  },
                  child: Text(
                    '取消',
                    style: TextStyle(fontSize: ScreenAdaper.height(25)),
                  )),
              const Spacer(),
              TextButton(
                  onPressed: () {
                    RouterUtil.back();
                    final year = 2000 + yearController.selectedItem;
                    String month = '${monthController.selectedItem + 1}';
                    if (int.parse(month) < 10) {
                      month = '0$month';
                    }
                    String day = '${dayController.selectedItem + 1}';
                    if (int.parse(day) < 10) {
                      day = '0$day';
                    }
                    if (onDateTimeChanged != null) {
                      onDateTimeChanged!('$year-$month-$day');
                    }
                    LoggerUtil().info('$year-$month-$day');
                  },
                  child: Text(
                    '确定',
                    style: TextStyle(fontSize: ScreenAdaper.height(25)),
                  ))
            ],
          ),
          Expanded(
              child: Row(
            children: <Widget>[
              Expanded(
                child: CupertinoPicker(
                  scrollController: yearController,
                  itemExtent: ScreenAdaper.height(60),
                  onSelectedItemChanged: (int index) {
                    // 处理用户选择的年份
                  },
                  children: List<Widget>.generate(DateTime.now().year - 1999,
                      (int index) {
                    return Center(child: Text('${2000 + index}年'));
                  }),
                ),
              ),
              Expanded(
                child: CupertinoPicker(
                  scrollController: monthController,
                  itemExtent: ScreenAdaper.height(60),
                  onSelectedItemChanged: (int index) {
                    // 处理用户选择的月份
                  },
                  children: List<Widget>.generate(12, (int index) {
                    return Center(child: Text('${index + 1}月'));
                  }),
                ),
              ),
              Expanded(
                child: CupertinoPicker(
                  scrollController: dayController,
                  itemExtent: ScreenAdaper.height(60),
                  onSelectedItemChanged: (int index) {
                    // 处理用户选择的日期
                  },
                  children: List<Widget>.generate(31, (int index) {
                    return Center(child: Text('${index + 1}日'));
                  }),
                ),
              ),
            ],
          ))
        ],
      ),
    );
  }
}
